//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <inifiles.hpp>

#include "utils.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
/*custom exception class. Used here to simplify logging of any exception that
may occur by throwing it out to the Application->OnException handler.*/
__fastcall ECustomException::ECustomException(String sFunc, String sType, String sNewMessage)
    : Exception(sNewMessage)
{
    FCallingFunction = sFunc;
    FExceptionType = sType;
    FNewMessage = SortOutWin32ErrorMessage(sNewMessage);
    FTime = Now();
}
//---------------------------------------------------------------------------
String __fastcall ECustomException::FGetTime()
{
    return FormatDateTime("dd-mm-yy hh:mm:ss", FTime);
}
//---------------------------------------------------------------------------
String __fastcall ECustomException::ErrorString()
{
    return FExceptionType + " : " + FNewMessage;
}
//---------------------------------------------------------------------------
String __fastcall ECustomException::SortOutWin32ErrorMessage(String sMessage)
{
TReplaceFlags RepFlags;
String Tempry;

RepFlags = RepFlags << rfIgnoreCase << rfReplaceAll;

Tempry = AdjustLineBreaks(sMessage);
return StringReplace(Tempry, "\r\n", " ", RepFlags); //check order of \n\r?
}
//---------------------------------------------------------------------------
/*simple position of form to avoid using TForm.Align which
causes problems to owned controls when setting width before
form is shown*/
void __fastcall PosForm(TForm *sForm, TAlign sAlign)
{
sForm->Width = (Screen->Width / 2);
sForm->Height = Screen->Height;
sForm->Top = 0;
switch(sAlign)
    {
    case alLeft: sForm->Left = 0; break;
    case alRight: sForm->Left = (Screen->Width - sForm->Width); break;
    }
}
//---------------------------------------------------------------------------
/*the following logs errors which occur. In this application it is called only
from the Application->OnException handler below. The ini file produced will be
[Function]
Time=Exception Class, Message*/
void __fastcall LogError(Exception *sException)
{
TIniFile *MyIni = NULL;
String CurrPath(ExtractFilePath(ParamStr(0)) + "\\error.log");
ECustomException *TempException;

    try
        {
        MyIni = new TIniFile(CurrPath);
        TempException = dynamic_cast <ECustomException *> (sException);
        if(TempException)
            MyIni->WriteString(TempException->CallingFunction, TempException->Time,
                                TempException->ErrorString());
        else
            MyIni->WriteString("Unknown", FormatDateTime("dd-mm-yy hh:mm:ss", Now()),
                            String(sException->ClassName()) + " : " + sException->Message);
        }
    __finally
        {
        if(MyIni)
            delete MyIni;
        }
}
//---------------------------------------------------------------------------
/*simple helper to return the parent folder of the current folder. eg
assuming ParamStr returns d:\borland\delphi\projects\articles\robust.exe
GetParentFolder
would both return
d:\borland\delphi\projects\*/
String __fastcall GetParentFolder()
{
String Tempry;
    Tempry = ExtractFilePath(ParamStr(0));
    Tempry = ExcludeTrailingBackslash(Tempry);
    return ExtractFilePath(Tempry);
}
//---------------------------------------------------------------------------

